﻿using UnityEngine;
using System.Collections.Generic;

public class LevelConfig : ScriptableObject
{
    public bool IsTutorialLevel = false;
	public float EnvironmentStage = 0f;
	public int SuccessPoints = 1;
	public int FailurePoints = -3;
	public int PointsToPass = 15;
	public float PointsToRegress = 5f;
	public int MinimumTries = 1;
    public float MoveSpeed = 1f;
    public List<Actions> MoveSets;
    public float TimeAfterActionToReset = 0.1f;
    public AudioClip MusicLoop;

    public float MovesPerMinute { get; private set; }
    public List<Actions> Moves { get; private set; }
    public float DelayBetweenNpcs { get; private set; }

    public enum Actions
    {
        Nothing = 0,
        Up = 10,
        LeftPosition = 30,
        CenterPosition = 40,
        RightPosition = 50
    }

    public enum CubiclePosition
    {
        Left = 0,
        Center = 10,
        Right = 20
    }

    public void SetUpMoves()
    {
        MovesPerMinute = MovementManager.Instance.MusicBPM * MoveSpeed;

        Moves = new List<Actions>();

        float numberOfStartNothingsToAdd = 0f;
        float numberOfEndNothingsToAdd = 0f;
        if (Mathf.Approximately(MoveSpeed, 1f))
        {
            numberOfStartNothingsToAdd = 0f;
            numberOfEndNothingsToAdd = 12f;
        }
        if (Mathf.Approximately(MoveSpeed, 2f))
        {
            numberOfStartNothingsToAdd = 1f;
            numberOfEndNothingsToAdd = 11f;
        }
        if (Mathf.Approximately(MoveSpeed, 4f))
        {
            numberOfStartNothingsToAdd = 3f;
            numberOfEndNothingsToAdd = 9f;
        }

        for (int i = 0; i < MoveSets.Count; i += 4)
        {
            for (int j = 0; j < numberOfStartNothingsToAdd; j++)
            {
                Moves.Add(Actions.Nothing);
            }

            Moves.Add(MoveSets[i]);
            Moves.Add(MoveSets[i + 1]);
            Moves.Add(MoveSets[i + 2]);
            Moves.Add(MoveSets[i + 3]);

            for (int j = 0; j < numberOfEndNothingsToAdd; j++)
            {
                Moves.Add(Actions.Nothing);
            }
        }

        DelayBetweenNpcs = 60f / MovesPerMinute * 4f;
    }

    public LevelConfig DeepCopy()
    {
        LevelConfig result = ScriptableObject.CreateInstance<LevelConfig>();
        result.name = this.name + "(DeepCopy)";

        result.IsTutorialLevel = IsTutorialLevel;
        result.EnvironmentStage = EnvironmentStage;
        result.SuccessPoints = SuccessPoints;
        result.FailurePoints = FailurePoints;
        result.PointsToPass = PointsToPass;
        result.PointsToRegress = PointsToRegress;
        result.MinimumTries = MinimumTries;
        result.MovesPerMinute = MovesPerMinute;
        result.MoveSets = MoveSets;
        result.TimeAfterActionToReset = TimeAfterActionToReset;
        result.MusicLoop = MusicLoop;

        result.Moves = Moves;
        result.DelayBetweenNpcs = DelayBetweenNpcs;

        return result;
    }
}
