﻿using UnityEngine;
using System;
using System.Collections.Generic;

public class Notification
{
    public GameObject TheObject;
    public Action<object> TheAction;
    public Action TheSimpleAction;

    public Notification(GameObject theObject, Action<object> theAction, Action theSimpleAction)
    {
        TheObject = theObject;
        TheAction = theAction;
        TheSimpleAction = theSimpleAction;
    }

    public void send(object notificationObject)
    {
        if (TheAction != null)
            TheAction(notificationObject);
        if (TheSimpleAction != null)
            TheSimpleAction();
    }
}

public enum NotificationName
{
    Unpause = 0
}

public class NotificationServer
{
    private static NotificationServer sharedInstance = null;
    private Dictionary<NotificationName, List<Notification>> notifications = new Dictionary<NotificationName, List<Notification>>();

    public static NotificationServer Instance {
        get {
            if (sharedInstance == null)
            {
                sharedInstance = new NotificationServer();
            }
            return sharedInstance;
        }
    }

    public void ClearAllNotifications()
    {
        notifications.Clear();
    }

    public void AddObserver(GameObject owner, NotificationName notificationMessage, Action simpleNotificationAction)
    {
        AddObserver(owner, notificationMessage, null, simpleNotificationAction);
    }

    public void AddObserver(GameObject owner, NotificationName notificationMessage, Action<object> notificationAction)
    {
        AddObserver(owner, notificationMessage, notificationAction, null);
    }

    public void AddObserver(GameObject owner, NotificationName notificationMessage, Action<object> notificationAction, Action simpleNotificationAction)
    {
        if (!Application.isPlaying)
            return;

        if (notifications.ContainsKey(notificationMessage))
        {
            List<Notification> theList = notifications[notificationMessage];
            theList.Add(new Notification(owner, notificationAction, simpleNotificationAction));
        }
        else
        {
            List<Notification> newList = new List<Notification>();
            newList.Add(new Notification(owner, notificationAction, simpleNotificationAction));
            notifications[notificationMessage] = newList;
        }
    }

    public void RemoveObserver(GameObject obj, NotificationName notificationMessage)
    {
        if (!Application.isPlaying)
            return;

        if (!notifications.ContainsKey(notificationMessage))
            return;

        List<Notification> theList = notifications[notificationMessage];
        for (int i = theList.Count - 1; i >= 0; i--)
        {
            if (theList[i].TheObject == obj)
                notifications[notificationMessage].Remove(theList[i]);
        }
    }

    public void PostNotification(NotificationName notificationMessage)
    {
        if (!Application.isPlaying)
            return;

        PostNotification(notificationMessage, null);
    }

    public void PostNotification(NotificationName notificationMessage, object notificationObject)
    {
        if (!Application.isPlaying)
            return;

        if (!notifications.ContainsKey(notificationMessage))
            return;

        List<Notification> theList = notifications[notificationMessage];
        int length = theList.Count;
        for (int i = 0; i < length; i++)
        {
            theList[i].send(notificationObject);
        }
    }
}
