﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class NpcMover : Mover
{
    protected override void Start()
    {
        base.Start();

        MovementManager.Instance.OnDoNextAction += OnDoNextAction;
    }

    protected override void OnDestroy()
    {
        base.OnDestroy();

        MovementManager.Instance.OnDoNextAction -= OnDoNextAction;
    }

    void Update()
    {
        if ((Time.time - actionStartTime) >= MovementManager.Instance.CurrentLevel.TimeAfterActionToReset)
        {
            if (currentAction == LevelConfig.Actions.Up)
            {
                transform.localPosition = new Vector3(transform.localPosition.x, startPosition.y, transform.localPosition.z);
            }
        }
    }

    private void OnDoNextAction(LevelConfig.Actions action, LevelConfig.CubiclePosition previousPosition, LevelConfig.CubiclePosition newPosition, float cubicleObjectHitDirection)
    {
        StartCoroutine(DoNextAction(action, previousPosition, newPosition, cubicleObjectHitDirection));
    }

    private IEnumerator DoNextAction(LevelConfig.Actions action, LevelConfig.CubiclePosition previousPosition, LevelConfig.CubiclePosition newPosition, float cubicleObjectHitDirection)
    {
        float delay = Order * MovementManager.Instance.CurrentLevel.DelayBetweenNpcs;

        yield return new WaitForSeconds(delay);

        if (MovementManager.Instance.CurrentLevel.IsTutorialLevel && action != LevelConfig.Actions.Nothing)
        {
            StartTutorialShow(action, previousPosition);
        }

        if (action == LevelConfig.Actions.Up)
        {
            transform.position = new Vector3(transform.position.x, transform.position.y + GlobalSettings.Instance.MoveDistanceVertical, transform.position.z);
            currentAction = LevelConfig.Actions.Up;
            actionStartTime = Time.time;

            if (currentPosition == LevelConfig.CubiclePosition.Left)
            {
                SoundManager.Instance.PlayPrinterSound();
                StartHitCubicleObject(CubicleObjectLeft, cubicleObjectHitDirection);
            }
            if (currentPosition == LevelConfig.CubiclePosition.Center)
            {
                SoundManager.Instance.PlayCabinetSound();
                StartHitCubicleObject(CubicleObjectCenter, cubicleObjectHitDirection);
            }
            if (currentPosition == LevelConfig.CubiclePosition.Right)
            {
                SoundManager.Instance.PlayComputerSound();
                StartHitCubicleObject(CubicleObjectRight, cubicleObjectHitDirection);
            }
        }
        else if (action == LevelConfig.Actions.LeftPosition)
        {
            SoundManager.Instance.PlayMoveSound();

            StartMovePosition(-1f);

            currentPosition = LevelConfig.CubiclePosition.Left;
            currentAction = LevelConfig.Actions.LeftPosition;
            actionStartTime = Time.time;
        }
        else if (action == LevelConfig.Actions.CenterPosition)
        {
            SoundManager.Instance.PlayMoveSound();

            if (currentPosition == LevelConfig.CubiclePosition.Left)
                StartMovePosition(1f);
            else
                StartMovePosition(-1f);
            
            currentPosition = LevelConfig.CubiclePosition.Center;
            currentAction = LevelConfig.Actions.CenterPosition;
            actionStartTime = Time.time;
        }
        else if (action == LevelConfig.Actions.RightPosition)
        {
            SoundManager.Instance.PlayMoveSound();

            StartMovePosition(1f);

            currentPosition = LevelConfig.CubiclePosition.Right;
            currentAction = LevelConfig.Actions.RightPosition;
            actionStartTime = Time.time;
        }
    }
}
