﻿using UnityEngine;
using System.Collections;

public class PlayerMover : Mover
{
    public float SuccessWindow = 0.2f;

    private float targetStage;
    private bool doingAction;
    private int successes;
    private float animatedSuccesses = 0;
    private float animatedStage = 0;
    float cubicleObjectHitDirection = 1f;

    public GameObject ZenSprite;
    public GameObject MainCamera;
    public GameObject FilmBarTop;
    private float FilmBarTop_startpos = -9001;

    protected override void Start()
    {
        base.Start();

        MovementManager.Instance.OnDoNextAction += OnDoNextAction;
    }

    protected override void OnDestroy()
    {
        base.OnDestroy();

        MovementManager.Instance.OnDoNextAction -= OnDoNextAction;
    }

    void Update()
    {
        if (!doingAction && Input.GetButtonDown("Use Action"))
        {
            transform.position = new Vector3(transform.position.x, transform.position.y + GlobalSettings.Instance.MoveDistanceVertical, transform.position.z);
            currentAction = LevelConfig.Actions.Up;
            doingAction = true;
            actionStartTime = Time.time;

            if (currentPosition == LevelConfig.CubiclePosition.Left)
            {
                SoundManager.Instance.PlayPrinterSound();
                StartHitCubicleObject(CubicleObjectLeft, cubicleObjectHitDirection);
            }
            if (currentPosition == LevelConfig.CubiclePosition.Center)
            {
                SoundManager.Instance.PlayCabinetSound();
                StartHitCubicleObject(CubicleObjectCenter, cubicleObjectHitDirection);
            }
            if (currentPosition == LevelConfig.CubiclePosition.Right)
            {
                SoundManager.Instance.PlayComputerSound();
                StartHitCubicleObject(CubicleObjectRight, cubicleObjectHitDirection);
            }
        }
        else if (Input.GetButtonDown("Move Left"))
        {
            if (currentPosition != LevelConfig.CubiclePosition.Left)
            {
                SoundManager.Instance.PlayMoveSound();

                doingAction = true;
                actionStartTime = Time.time;

                StartMovePosition(-1f);

                if (currentPosition == LevelConfig.CubiclePosition.Center)
                {
                    currentAction = LevelConfig.Actions.LeftPosition;
                    currentPosition = LevelConfig.CubiclePosition.Left;
                }
                else if (currentPosition == LevelConfig.CubiclePosition.Right)
                {
                    currentAction = LevelConfig.Actions.CenterPosition;
                    currentPosition = LevelConfig.CubiclePosition.Center;
                }
            }
        }
        else if (Input.GetButtonDown("Move Right"))
        {
            if (currentPosition != LevelConfig.CubiclePosition.Right)
            {
                SoundManager.Instance.PlayMoveSound();

                doingAction = true;
                actionStartTime = Time.time;

                StartMovePosition(1f);

                if (currentPosition == LevelConfig.CubiclePosition.Center)
                {
                    currentAction = LevelConfig.Actions.RightPosition;
                    currentPosition = LevelConfig.CubiclePosition.Right;
                }
                else if (currentPosition == LevelConfig.CubiclePosition.Left)
                {
                    currentAction = LevelConfig.Actions.CenterPosition;
                    currentPosition = LevelConfig.CubiclePosition.Center;
                }
            }
        }
        else if (doingAction && (Time.time - actionStartTime) >= MovementManager.Instance.CurrentLevel.TimeAfterActionToReset)
        {
            if (currentAction == LevelConfig.Actions.Up)
            {
                transform.localPosition = new Vector3(transform.localPosition.x, startPosition.y, transform.localPosition.z);
            }
            doingAction = false;
        }
    }

    private void OnDoNextAction(LevelConfig.Actions action, LevelConfig.CubiclePosition previousPosition, LevelConfig.CubiclePosition newPosition, float cubicleObjectHitDirection)
    {
        this.cubicleObjectHitDirection = cubicleObjectHitDirection;

        if (MovementManager.Instance.CurrentLevel.IsTutorialLevel && action != LevelConfig.Actions.Nothing)
        {
            StartCoroutine(ShowTutorial(action, previousPosition));
        }

        StartCoroutine(DoNextAction(action, previousPosition, newPosition, cubicleObjectHitDirection));
    }

    private IEnumerator DoNextAction(LevelConfig.Actions action, LevelConfig.CubiclePosition previousPosition, LevelConfig.CubiclePosition newPosition, float cubicleObjectHitDirection)
    {
        float delay = Order * MovementManager.Instance.CurrentLevel.DelayBetweenNpcs;

        yield return new WaitForSeconds(delay - SuccessWindow / 2f);

        float successCheckTimeStart = Time.time;
        bool hasSucceeded = false;

        while ((Time.time - successCheckTimeStart) < SuccessWindow)
        {
            if (action == LevelConfig.Actions.Nothing)
            {
                // Don't score for Nothing as Nothing is meant to just be used a gap in the beats
                break;
            }
            else if (action == LevelConfig.Actions.Up)
            {
                // If current action is an up action then we need to care about timing (doingAction),
                // doing the right action, and being in the right position
                if (doingAction && currentAction == action && currentPosition == newPosition)
                {
                    hasSucceeded = true;
                    break;
                }
            }
            else
            {
                // If current action is a move left/right action then don't worry about timing
                if (currentPosition == newPosition)
                {
                    hasSucceeded = true;
                    break;
                }
            }

            yield return new WaitForEndOfFrame();
        }

        if (hasSucceeded)
        {
            UpdateScore(MovementManager.Instance.CurrentLevel.SuccessPoints);
        }
        else if (action != LevelConfig.Actions.Nothing)
        {
            UpdateScore(MovementManager.Instance.CurrentLevel.FailurePoints);
        }
    }

    private IEnumerator ShowTutorial(LevelConfig.Actions action, LevelConfig.CubiclePosition previousPosition)
    {
        float delay = Order * MovementManager.Instance.CurrentLevel.DelayBetweenNpcs;

        yield return new WaitForSeconds(delay);

        if (action == LevelConfig.Actions.Up)
            StartTutorialShow(action, previousPosition);
        if (action == LevelConfig.Actions.LeftPosition)
            StartTutorialShow(action, LevelConfig.CubiclePosition.Center);
        if (action == LevelConfig.Actions.CenterPosition)
            StartTutorialShow(action, previousPosition);
        if (action == LevelConfig.Actions.RightPosition)
            StartTutorialShow(action, LevelConfig.CubiclePosition.Center);
    }

    public float AnimatedStage { get { return animatedStage; } }
    public float TargetStage { set {
            if (targetStage == value)
                return;
            targetStage = value;
            if (!animating)
            {
                animating = true;
                StartCoroutine(AnimateScore());
            }
        }
    }

    public int Points { get { return successes; } }
    public float Energy { get { return animatedSuccesses / MovementManager.Instance.MaximumPoints; } }

    private bool animating = false;
    private void UpdateScore(int change)
    {
        successes += change;
        successes = successes < 0 ? 0 : successes;
        successes = successes > MovementManager.Instance.MaximumPoints ? MovementManager.Instance.MaximumPoints : successes;
        if (!animating)
        {
            animating = true;
            StartCoroutine(AnimateScore());
        }
    }

    private IEnumerator AnimateScore()
    {
        while (animatedSuccesses != successes || animatedStage != targetStage)
        {
            if (animatedSuccesses != successes)
            {
                float speed = Time.deltaTime * 0.5f + Mathf.Abs(animatedSuccesses - successes) * Time.deltaTime * 2.0f;
                if (animatedSuccesses < successes)
                {
                    animatedSuccesses += speed;
                    if (animatedSuccesses > successes)
                        animatedSuccesses = successes;
                }
                else
                {
                    animatedSuccesses -= speed;
                    if (animatedSuccesses < successes)
                        animatedSuccesses = successes;
                }

                float energy = Energy;
                float energy1 = Mathf.Clamp01(energy * 2);
                float energy2 = Mathf.Clamp01(energy * 2 - 1);
                float energy3 = Mathf.Clamp01(energy2 * 2);

                if (MainCamera != null)
                {
                    ImageEffects.EnergyDesaturate e = MainCamera.GetComponent<ImageEffects.EnergyDesaturate>();
                    if (e != null)
                        e.GetMaterial().SetFloat("_energy", energy1);
                }
                if (ZenSprite != null)
                {
                    SpriteRenderer r = ZenSprite.GetComponent<SpriteRenderer>();
                    if (r != null)
                        r.material.SetFloat("_energy", energy2);
                }
                if (FilmBarTop != null)
                {
                    if (FilmBarTop_startpos < -9000)
                        FilmBarTop_startpos = FilmBarTop.transform.position.y;

                    Vector3 p = FilmBarTop.transform.position;
                    p.y = FilmBarTop_startpos + energy3 * 2.0f;
                    FilmBarTop.transform.position = p;
                }
            }
            if (animatedStage != targetStage)
            {
                float speed = Time.deltaTime * 0.1f + Mathf.Abs(animatedStage - targetStage) * Time.deltaTime * 0.05f;
                if (animatedStage < targetStage)
                {
                    animatedStage += speed;
                    if (animatedStage > targetStage)
                        animatedStage = targetStage;
                }
                else
                {
                    animatedStage -= speed;
                    if (animatedStage < targetStage)
                        animatedStage = targetStage;
                }

                if (ZenSprite != null)
                {
                    SpriteRenderer r = ZenSprite.GetComponent<SpriteRenderer>();
                    if (r != null)
                        r.material.SetFloat("_stage", animatedStage);
                }
            }

            yield return new WaitForEndOfFrame();
        }
        animating = false;
    }
}
