﻿using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;
using System.Collections;

public class GameOverScene : MonoBehaviour
{
    public Fader FaderSprite;
    public AudioSource Ambient;
    public float AmbientFadeTime = 0.5f;

    private float ambientVolumeLow = 0.01f;
    private float ambientVolumeHigh;

    void Start()
    {
        FaderSprite.StartFadeIn();

        ambientVolumeHigh = Ambient.volume;
        Ambient.volume = ambientVolumeLow;
        Ambient.Play();
        StartCoroutine(DoAmbientFade(ambientVolumeLow, ambientVolumeHigh));
    }

    public void ButtonRestart()
    {
        StartCoroutine(Restart());
    }

    public void ButtonMainMenu()
    {
        StartCoroutine(MainMenu());
    }

    private IEnumerator Restart()
    {
        // Start the scene load but don't allow it to auto finish loading until we say so
        AsyncOperation async = SceneManager.LoadSceneAsync(2);
        async.allowSceneActivation = false;

        StartCoroutine(DoAmbientFade(ambientVolumeHigh, ambientVolumeLow));

        FaderSprite.StartFadeOut();

        yield return new WaitForSeconds(FaderSprite.FadeTime * 1.5f);

        // We've finished showing what we wanted and then fading so allow Unity to finish the scene load
        async.allowSceneActivation = true;
    }

    private IEnumerator MainMenu()
    {
        // Start the scene load but don't allow it to auto finish loading until we say so
        AsyncOperation async = SceneManager.LoadSceneAsync(1);
        async.allowSceneActivation = false;

        StartCoroutine(DoAmbientFade(ambientVolumeHigh, ambientVolumeLow));

        FaderSprite.StartFadeOut();

        yield return new WaitForSeconds(FaderSprite.FadeTime * 1.5f);

        // We've finished showing what we wanted and then fading so allow Unity to finish the scene load
        async.allowSceneActivation = true;
    }

    private IEnumerator DoAmbientFade(float volumnFrom, float volumnTo)
    {
        float startTime = Time.time;

        while ((Time.time - startTime) <= AmbientFadeTime)
        {
            Ambient.volume = Mathf.Lerp(volumnFrom, volumnTo, (Time.time - startTime) / AmbientFadeTime);

            yield return null;
        }

        Ambient.volume = volumnTo;

        yield return null;
    }
}
