﻿using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;
using System.Collections;

public class GameScene : MonoBehaviour
{
    public Fader FaderSprite;
    public Image PauseButton;
    public Sprite PauseSprite;
    public Sprite PlaySprite;
    public GameObject MainMenuButton;

    private bool currentlyPaused = false;

    void Start()
    {
        FaderSprite.StartFadeIn();

        PauseButton.sprite = PauseSprite;
        MainMenuButton.SetActive(false);
    }

    public void ButtonPause()
    {
        if (currentlyPaused)
        {
            PauseButton.sprite = PauseSprite;
            MainMenuButton.SetActive(false);
            Time.timeScale = 1f;
            NotificationServer.Instance.PostNotification(NotificationName.Unpause);
        }
        else
        {
            PauseButton.sprite = PlaySprite;
            MainMenuButton.SetActive(true);
            SoundManager.Instance.Pause();
            Time.timeScale = 0f;
        }

        currentlyPaused = !currentlyPaused;
    }

    public void ButtonLoadMainMenu()
    {
        Time.timeScale = 1f;

        StartCoroutine(LoadMainMenu());
    }

    public void StartLoadGameOver()
    {
        SoundManager.Instance.FadeOutAll();

        StartCoroutine(LoadGameOver());
    }

    private IEnumerator LoadMainMenu()
    {
        // Start the scene load but don't allow it to auto finish loading until we say so
        AsyncOperation async = SceneManager.LoadSceneAsync(1);
        async.allowSceneActivation = false;

        FaderSprite.StartFadeOut();

        yield return new WaitForSeconds(FaderSprite.FadeTime * 1.5f);

        // We've finished showing what we wanted and then fading so allow Unity to finish the scene load
        async.allowSceneActivation = true;
    }

    private IEnumerator LoadGameOver()
    {
        // Start the scene load but don't allow it to auto finish loading until we say so
        AsyncOperation async = SceneManager.LoadSceneAsync(4);
        async.allowSceneActivation = false;

        FaderSprite.StartFadeOut();

        yield return new WaitForSeconds(FaderSprite.FadeTime * 1.5f);

        // We've finished showing what we wanted and then fading so allow Unity to finish the scene load
        async.allowSceneActivation = true;
    }
}
