﻿using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;
using System.Collections;

public class LoadingScene : MonoBehaviour
{
    public Fader FaderSprite;
    public float MinTimeToStayInScene = 2f;
    public float TimeBeforeStartLoad = 0.5f;

    IEnumerator Start()
    {
        FaderSprite.StartFadeIn();

        // Wait before starting to load next scene so that Unity has a chance to finish loading the current scene
        yield return new WaitForSeconds(TimeBeforeStartLoad);

        // Start the scene load but don't allow it to auto finish loading until we say so
        AsyncOperation async = SceneManager.LoadSceneAsync(1);
        async.allowSceneActivation = false;

        yield return new WaitForSeconds(MinTimeToStayInScene - TimeBeforeStartLoad - FaderSprite.FadeTime);

        FaderSprite.StartFadeOut();

        yield return new WaitForSeconds(FaderSprite.FadeTime * 1.5f);

        // We've finished showing what we wanted and then fading so allow Unity to finish the scene load
        async.allowSceneActivation = true;
    }
}
