﻿using UnityEngine;
using System.Collections;

public class SoundManager : MonoBehaviour
{
    public static SoundManager Instance;
	
    public float FadeOutTime = 0.5f;
    public AudioSource BackgroundBeat1;
    public AudioSource BackgroundBeat2;
    public AudioSource MoveSound;
    public AudioSource PrinterSound;
    public AudioSource CabinetSound;
    public AudioSource ComputerSound;
    public AudioSource Loop1;
    public AudioSource Loop2;

    private AudioClip PrinterSoundOrig;
    private AudioClip CabinetSoundOrig;
    private AudioClip ComputerSoundOrig;
    public AudioClip PrinterSoundAlt;
    public AudioClip CabinetSoundAlt;
    public AudioClip ComputerSoundAlt;

    private float pauseBackgroundBeat1 = -1f;
    private float pauseBackgroundBeat2 = -1f;
    private float pauseMoveSound = -1f;
    private float pausePrinterSound = -1f;
    private float pauseCabinetSound = -1f;
    private float pauseComputerSound = -1f;
    private float pauseLoop1 = -1f;
    private float pauseLoop2 = -1f;

    void Awake()
    {
        Instance = this;

        PrinterSoundOrig = PrinterSound.clip;
        CabinetSoundOrig = CabinetSound.clip;
        ComputerSoundOrig = ComputerSound.clip;
    }

    public void Pause()
    {
        pauseBackgroundBeat1 = -1f;
        pauseBackgroundBeat2 = -1f;
        pauseMoveSound = -1f;
        pausePrinterSound = -1f;
        pauseCabinetSound = -1f;
        pauseComputerSound = -1f;
        pauseLoop1 = -1f;
        pauseLoop2 = -1f;

        if (BackgroundBeat1.isPlaying)
        {
            pauseBackgroundBeat1 = BackgroundBeat1.time;
            BackgroundBeat1.Stop();
        }

        if (BackgroundBeat2.isPlaying)
        {
            pauseBackgroundBeat2 = BackgroundBeat2.time;
            BackgroundBeat2.Stop();
        }

        if (MoveSound.isPlaying)
        {
            pauseMoveSound = MoveSound.time;
            MoveSound.Stop();
        }

        if (PrinterSound.isPlaying)
        {
            pausePrinterSound = PrinterSound.time;
            PrinterSound.Stop();
        }

        if (CabinetSound.isPlaying)
        {
            pauseCabinetSound = CabinetSound.time;
            CabinetSound.Stop();
        }

        if (ComputerSound.isPlaying)
        {
            pauseComputerSound = ComputerSound.time;
            ComputerSound.Stop();
        }

        if (Loop1.isPlaying)
        {
            pauseLoop1 = Loop1.time;
            Loop1.Stop();
        }

        if (Loop2.isPlaying)
        {
            pauseLoop2 = Loop2.time;
            Loop2.Stop();
        }
    }

    // This function doesn't work well at all so instead of using this I'm just going to restart to the start of a level
    public void Resume()
    {
        if (!Mathf.Approximately(pauseBackgroundBeat1, -1f))
        {
            BackgroundBeat1.Play();
            BackgroundBeat1.time = pauseBackgroundBeat1;
        }

        if (!Mathf.Approximately(pauseBackgroundBeat2, -1f))
        {
            BackgroundBeat2.Play();
            BackgroundBeat2.time = pauseBackgroundBeat2;
        }

        if (!Mathf.Approximately(pauseMoveSound, -1f))
        {
            MoveSound.time = pauseMoveSound;
            MoveSound.Play();
        }

        if (!Mathf.Approximately(pausePrinterSound, -1f))
        {
            PrinterSound.time = pausePrinterSound;
            PrinterSound.Play();
        }

        if (!Mathf.Approximately(pauseCabinetSound, -1f))
        {
            CabinetSound.time = pauseCabinetSound;
            CabinetSound.Play();
        }

        if (!Mathf.Approximately(pauseComputerSound, -1f))
        {
            ComputerSound.time = pauseComputerSound;
            ComputerSound.Play();
        }

        if (!Mathf.Approximately(pauseLoop1, -1f))
        {
            Loop1.Play();
            Loop1.time = pauseLoop1;
        }

        if (!Mathf.Approximately(pauseLoop2, -1f))
        {
            Loop2.Play();
            Loop2.time = pauseLoop2;
        }
    }

    public void FadeOutAll()
    {
        StartCoroutine(DoFadeOut(BackgroundBeat1));
        StartCoroutine(DoFadeOut(BackgroundBeat2));
        StartCoroutine(DoFadeOut(MoveSound));
        StartCoroutine(DoFadeOut(PrinterSound));
        StartCoroutine(DoFadeOut(CabinetSound));
        StartCoroutine(DoFadeOut(ComputerSound));
        StartCoroutine(DoFadeOut(Loop1));
        StartCoroutine(DoFadeOut(Loop2));
    }

    private IEnumerator DoFadeOut(AudioSource audio)
    {
        float volumeFrom = audio.volume;
        float volumeTo = 0f;
        float startTime = Time.time;

        while ((Time.time - startTime) <= FadeOutTime)
        {
            audio.volume = Mathf.Lerp(volumeFrom, volumeTo, (Time.time - startTime) / FadeOutTime);

            yield return null;
        }

        audio.volume = 0f;
    }

    private void CheckStage()
    {
        if (MovementManager.Instance.CurrentLevel.EnvironmentStage > 1 && PrinterSound.clip != PrinterSoundAlt)
        {
            PrinterSound.clip = PrinterSoundAlt; 
            CabinetSound.clip = CabinetSoundAlt;
            ComputerSound.clip = ComputerSoundAlt;
        }
        else if (MovementManager.Instance.CurrentLevel.EnvironmentStage <= 1 && PrinterSound.clip != PrinterSoundOrig)
        {
            PrinterSound.clip = PrinterSoundOrig;
            CabinetSound.clip = CabinetSoundOrig;
            ComputerSound.clip = ComputerSoundOrig;
        }
    }

    public void PlayBackgroundBeat1()
    {
        BackgroundBeat1.Play();
    }

    public void PlayBackgroundBeat2()
    {
        BackgroundBeat2.Play();
    }

    public void PlayMoveSound()
    {
        MoveSound.Play();
    }

    public void PlayPrinterSound()
    {
        CheckStage();
        PrinterSound.Play();
    }

    public void PlayCabinetSound()
    {
        CheckStage();
        CabinetSound.Play();
    }

    public void PlayComputerSound()
    {
        CheckStage();
        ComputerSound.Play();
    }

    public void PlayLoop1(AudioClip loopClip)
    {
        if (Loop1.clip != loopClip)
            Loop1.clip = loopClip;
        
        Loop1.Play();
    }

    public void PlayLoop2(AudioClip loopClip)
    {
        if (Loop2.clip != loopClip)
            Loop2.clip = loopClip;
        
        Loop2.Play();
    }
}
