﻿using UnityEngine;
using System.Collections;

public class SpriteSwapper : MonoBehaviour {

	private float animationTimer = -1;
	private Sprite originalSprite;
	public Sprite alternativeSprite;
	private Sprite animatingTo;

	public bool IsAnimating { get { return animationTimer >= 0; } }

	void Awake()
	{
		var r = GetComponent<SpriteRenderer>();
		originalSprite = r.sprite;
	}

	private float Stage
	{
		get
		{
//			return 1;
			float xMin = -9, xMax = 9;
			float xFract = (gameObject.transform.position.x - xMin) / (xMax - xMin);
			return 1 + Mathf.Clamp01(xFract) * GlobalSettings.Instance.MexicanZenWave;
		}
	}
	private float TargetStage
	{
		get
		{
			float stage = MovementManager.Instance.CurrentLevel.EnvironmentStage;
			var pm = MovementManager.Instance.Player.GetComponent<PlayerMover>();
			if (pm != null)
				stage = pm.AnimatedStage;
			return stage;
		}
	}

	void Update () 
	{
		if (animationTimer >= 0)
			return;
		
		var r = GetComponent<SpriteRenderer>();
		bool isALt = (r.sprite == alternativeSprite);
		float stage = Stage;
		float targetStage = TargetStage;

		if (targetStage > stage && !isALt)
		{
			animationTimer = 0;
			animatingTo = alternativeSprite;
			StartCoroutine(AnimateTransition());
		}
		else if (targetStage < stage && isALt)
		{
			animationTimer = 0;
			animatingTo = originalSprite;
			StartCoroutine(AnimateTransition());
		}
	}
	private IEnumerator AnimateTransition()
	{
		var r = GetComponent<SpriteRenderer>();
		float transitionTime = GlobalSettings.Instance.TransitionTotal;
		float switchAt = GlobalSettings.Instance.TransitionSwitch;
		while (animationTimer < transitionTime)
		{
			float prevTimer = animationTimer;
			animationTimer += Time.deltaTime;

			if (prevTimer < switchAt && animationTimer >= switchAt)
				r.sprite = animatingTo;

			float s = 1;
			if (animationTimer <= switchAt)
				s = 1 - (animationTimer / switchAt);
			else
				s = (animationTimer - switchAt)/(transitionTime - switchAt);
			s = s * s * (3 - 2 * s);
			Vector3 scale = new Vector3(s, s, s);
			gameObject.transform.localScale = scale;

			yield return new WaitForEndOfFrame();
		}
		animationTimer = -1;
	}
}
