﻿using UnityEngine;
using UnityEngine.UI;

public class Fader : MonoBehaviour
{
    public float FadeTime = 0.5f;
    public Image FadeSprite;

    private bool fading;
    private bool fadingIn;
    private float fadeStartTime;

    void Update()
    {
        if (fading)
        {
            float alpha = 0f;

            if (fadingIn)
            {
                alpha = Mathf.Lerp(1f, 0f, (Time.time - fadeStartTime) / FadeTime);

                if ((Time.time - fadeStartTime) >= FadeTime)
                {
                    // Make sure to set the alpha value to the final value as the lerp might not have set all the way to the final value
                    alpha = 0f;
                    fading = false;
                }
            }
            else
            {
                alpha = Mathf.Lerp(0f, 1f, (Time.time - fadeStartTime) / FadeTime);

                if ((Time.time - fadeStartTime) >= FadeTime)
                {
                    // Make sure to set the alpha value to the final value as the lerp might not have set all the way to the final value
                    alpha = 1f;
                    fading = false;
                }
            }

            FadeSprite.color = new Color(FadeSprite.color.r, FadeSprite.color.g, FadeSprite.color.b, alpha);
        }
    }

    public void StartFadeIn()
    {
        // The process of fading in is simply taking a full screen image from alpha=1 to alpha=0
        FadeSprite.color = new Color(FadeSprite.color.r, FadeSprite.color.g, FadeSprite.color.b, 1f);
        fadeStartTime = Time.time;
        fadingIn = true;
        fading = true;
    }

    public void StartFadeOut()
    {
        // The process of fading out is simply taking a full screen image from alpha=0 to alpha=1
        FadeSprite.color = new Color(FadeSprite.color.r, FadeSprite.color.g, FadeSprite.color.b, 0f);
        fadeStartTime = Time.time;
        fadingIn = false;
        fading = true;
    }
}
