﻿using UnityEngine;

public class UIPieceMover : MonoBehaviour
{
    public Vector3 StartMovePosition;
    public float DelayTime = 1f;
    public float MoveTime = 2f;
    public AudioSource SoundAtStartToPlay;
    public AudioSource SoundAtEndToPlay;

    private RectTransform rect;
    private float moveStartTime;
    private Vector3 endMovePosition;
    private bool soundAtStartPlayed = false;

    void Start()
    {
        rect = GetComponent<RectTransform>();
        endMovePosition = rect.localPosition;
        rect.localPosition = StartMovePosition;
        moveStartTime = Time.time;
    }
	
    void Update()
    {
        if ((Time.time - moveStartTime) <= DelayTime)
            return;

        if (!soundAtStartPlayed && SoundAtStartToPlay != null)
        {
            SoundAtStartToPlay.Play();
            soundAtStartPlayed = true;
        }
        
        if ((Time.time - moveStartTime) <= (DelayTime + MoveTime))
        {
            rect.localPosition = Vector3.Lerp(StartMovePosition, endMovePosition, Mathf.SmoothStep(0f, 1f, (Time.time - moveStartTime - DelayTime) / (MoveTime - DelayTime)));
        }
        else
        {
            if (SoundAtEndToPlay != null)
                SoundAtEndToPlay.Play();

            rect.localPosition = endMovePosition;
            this.enabled = false;
        }
    }
}
