Shader "Sprites/GGJ"
{
	Properties
	{
		[PerRendererData] _MainTex ("Sprite Texture", 2D) = "white" {}
		_Color ("Tint", Color) = (1,1,1,1)
		_MaskTex ("Mask", 2D) = "white" {}
		[MaterialToggle] PixelSnap ("Pixel snap", Float) = 0
	}

	SubShader
	{
		Tags
		{ 
			"Queue"="Transparent" 
			"IgnoreProjector"="True" 
			"RenderType"="Transparent" 
			"PreviewType"="Plane"
			"CanUseSpriteAtlas"="True"
		}

		Cull Off
		Lighting Off
		ZWrite Off
		Blend One OneMinusSrcAlpha

		Pass
		{
		CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			#pragma multi_compile _ PIXELSNAP_ON
			#include "UnityCG.cginc"
			
			struct appdata_t
			{
				float4 vertex   : POSITION;
				float4 color    : COLOR;
				float2 texcoord : TEXCOORD0;
			};

			struct v2f
			{
				float4 vertex   : SV_POSITION;
				fixed4 color    : COLOR;
				float2 texcoord  : TEXCOORD0;
				float2 texcoord2 : TEXCOORD1;
			};
			
			fixed4 _Color;

			v2f vert(appdata_t IN)
			{
				v2f OUT;
				OUT.vertex = mul(UNITY_MATRIX_MVP, IN.vertex);
				OUT.texcoord = IN.texcoord;
				OUT.color = IN.color * _Color;
				#ifdef PIXELSNAP_ON
				OUT.vertex = UnityPixelSnap (OUT.vertex);
				#endif

				OUT.texcoord2 = OUT.vertex.xy * 0.5+0.5;
				OUT.texcoord2 *= float2(16,9);
				OUT.texcoord2/=9;
//				OUT.texcoord2.x *= 4;

	//			OUT.vertex.x += cos(_Time.w*3) * 0.1 * saturate(OUT.vertex.y*0.5+0.5-0.3);

				return OUT;
			}

			sampler2D _MainTex;
			sampler2D _MaskTex;
			sampler2D _AlphaTex;
			float _AlphaSplitEnabled;

			fixed4 SampleSpriteTexture (float2 uv)
			{
				fixed4 color = tex2D (_MainTex, uv);

#if UNITY_TEXTURE_ALPHASPLIT_ALLOWED
				if (_AlphaSplitEnabled)
					color.a = tex2D (_AlphaTex, uv).r;
#endif //UNITY_TEXTURE_ALPHASPLIT_ALLOWED

				return color;
			}

			fixed4 frag(v2f IN) : SV_Target
			{
				fixed4 c = SampleSpriteTexture (IN.texcoord) * IN.color;
				
				fixed d = dot(c.rgb, 1/3.0);
				
				fixed4 msk = tex2D(_MaskTex, IN.texcoord2);

				float testDuration = 1.0/8;
				float pulses = 4;
				float pulseSize = 0.2;
				float energy = saturate(frac(_Time.y*-testDuration)*1.2-0.1);
				energy = saturate( energy + saturate(sin(energy*2*3.14159*pulses))*pulseSize );
			//	energy = 1-saturate((1-energy)*3);

				msk.r = (msk.r+d)/2;
				
				float hardness = 3;
				float mask = saturate( (msk.r - energy)*hardness );
				c.rgb = lerp( (d).xxx, c.rgb, mask);

				c.rgb *= c.a;
				return c;
			}
		ENDCG
		}
	}
}
